/*
This is the Gibbs sampler for the dengue fever dataset analyzed in Fernandes, Schmidt and Migon "Modelling zero-inflated spatio-temporal processes".
Notice that the fixed parameters are sampled via a ARM step. Therefore, one needs the library arms.h to run this code.
*/

#include <oxstd.h>   
#include <oxprob.h>	 
#include <oxfloat.h> 
#include <oxdraw.h>
#include "arms.h"

// gama0, gama1 and alpha0 are sampled via ARMS steps (Wild and Gilks)


decl g_gama0data;
decl g_gama1data;
decl g_alfa0data;

class Testedata
{
decl mig0,sigg0,I, I2,a0, g0, g1, Z1, S1, tem, ee, n1, T1, obs, mig1, sigg1, mia0, siga0;
log_lik(const x1);
log_lik2(const x2);
log_lik3(const x3);
set_mig0(migama0);
set_sigg0(sigmagama0);
set_mig1(migama1);
set_sigg1(sigmagama1);
set_mia0(mialfa0);
set_siga0(sigmaalfa0);
set_I(Ind);
set_I2(Ind2);
set_a0(alfa0);
set_g0(gama0);
set_g1(gama1);
set_Z1(Z);
set_T1(T);
set_n1(n);
set_tem(temp1);
set_ee(e);
set_S1(S);
set_obs(Y);
Estimate(const ninit, const mxl, const mxr, const domedrop, const xprev, const mxsamp);
}

Testedata::log_lik(const x1)
{
return (-(1/(2*sigg0))*(x1-mig0)^2) + x1*sumr(sumc(tem)) - sumr(sumc(log(1 + exp(x1 + g1*I2))));
}

Testedata::log_lik2(const x2)
{
return (-(1/(2*sigg1))*(x2-mig1)^2) + x2*sumr(sumc(tem.*I2)) - sumr(sumc(log(1 + exp(g0 + x2*I2))));
}

Testedata::log_lik3(const x3)
{
return (-(1/(2*siga0))*(x3-mia0)^2) + x3*sumc(sumr(tem.*obs)) - sumr(sumc(tem.*exp(x3+S1).*ee));
}

Testedata::set_mig0(migama0)
{mig0 = migama0;}
Testedata::set_sigg0(sigmagama0)
{sigg0 = sigmagama0;}
Testedata::set_mig1(migama1)
{mig1 = migama1;}
Testedata::set_sigg1(sigmagama1)
{sigg1 = sigmagama1;}
Testedata::set_mia0(mialfa0)
{mia0 = mialfa0;}
Testedata::set_siga0(sigmaalfa0)
{siga0 = sigmaalfa0;}
Testedata::set_I(Ind)
{I = Ind;}
Testedata::set_I2(Ind2)
{I2 = Ind2;}
Testedata::set_g0(gama0)
{g0 = gama0;}
Testedata::set_a0(alfa0)
{a0 = alfa0;}
Testedata::set_g1(gama1)
{g1 = gama1;}
Testedata::set_Z1(Z)
{Z1 =Z;}
Testedata::set_T1(T)
{T1 = T;}
Testedata::set_n1(n)
{n1 = n;}
Testedata::set_tem(temp1)
{tem = temp1;}
Testedata::set_ee(e)
{ee = e;}
Testedata::set_S1(S)
{S1 = S;}
Testedata::set_obs(Y)
{obs = Y;}

log_lik(const x1)
{
return g_gama0data -> log_lik(x1);
}

log_lik2(const x2)
{
return g_gama1data -> log_lik2(x2);
}

log_lik3(const x3)
{
return g_alfa0data -> log_lik3(x3);
}





main ()
{
ranseed(98639963246);

// building the neighborhood matrix W

decl Viz = new array [156] ;

Viz[0]=<1,4>;
Viz[1]=<0,2,4>;
Viz[2]=<1,3,4,7,9,30>;
Viz[3]=<2,9,11,37,103,155>;
Viz[4]=<0,1,2,7,12,14>;
Viz[5]=<6,7,8,12>;	
Viz[6]=<5,8,12,30,31,32>;
Viz[7]=<2,4,5,8,12,30>;
Viz[8]=<5,6,7,30>;
Viz[9]=<2,3,10,11,30,33>;
Viz[10]=<9,11,33,34,55>;
Viz[11]=<3,9,10,37,49,55,56,153>;
Viz[12]=<4,5,6,7,14,15,16,17,18,19,32>;
Viz[13]=<14,15,16,18>;	
Viz[14]=<4,12,13,16>;
Viz[15]=<12,13,16,17,18>;
Viz[16]=<12,13,14,15>;
Viz[17]=<12,15,18>;
Viz[18]=<12,13,15,17,19,20,21,22,25>;
Viz[19]=<12,18,22,25,26,32>;
Viz[20]=<18,21,22>;
Viz[21]=<18,20,22>;
Viz[22]=<18,19,20,21,23,25>;
Viz[23]=<22,24,25>;
Viz[24]=<23,25,27,28>;
Viz[25]=<18,19,22,23,24,26,27>;
Viz[26]=<19,25,27,32>;	
Viz[27]=<24,25,26,28,29,32,152>;
Viz[28]=<24,27,29,152>;
Viz[29]=<27,28,32,124,125,126,152>;
Viz[30]=<2,6,7,8,9,31,33>;
Viz[31]=<6,30,32,33,34,35,36>;
Viz[32]=<6,12,19,26,27,29,31,36,113,125>;
Viz[33]=<9,10,30,31,34,55>;
Viz[34]=<10,31,33,35,36,55,56,57,58,59>;
Viz[35]=<31,34,36>;
Viz[36]=<31,32,34,35,59,60,113>;
Viz[37]=<3,11,38,48,153,155>;
Viz[38]=<37,39,48,154,155>;
Viz[39]=<38,40,154,155>;
Viz[40]=<39,41,42,53,154,155>;
Viz[41]=<40,42,53,155>;
Viz[42]=<40,41,43,44,53,70,72>;
Viz[43]=<42,44,72,73,74>;
Viz[44]=<42,43,45,73,74>;
Viz[45]=<44,46,73,74>;
Viz[46]=<45,47,74>;
Viz[47]=<46,74,112>;
Viz[48]=<37,38,50,51,52,153,154>;
Viz[49]=<11,56,57,58,59,63,153>;	
Viz[50]=<48,51,63,153>;	
Viz[51]=<48,50,52,63>;
Viz[52]=<48,51,	53,	63,	64,	69,	154	>;
Viz[53]=<40,41,	42,	52,	54,	69,	70,	154	>;
Viz[54]=<53,67,	69,	70,	71,	78>;
Viz[55]=<10,11,	33,	34,	56>;
Viz[56]=<11,34,	49,	55,	57>;
Viz[57]=<34,49,	56,	58>;
Viz[58]=<34,49,	57,	59>;
Viz[59]=<34,36,	49,	58,	60,	61,	63>;
Viz[60]=<36,59,	61,	64,	113	>;		
Viz[61]=<59,60,	62,	63,	64>;		
Viz[62]=<61,63,	64	>;																																	
Viz[63]=<49,50,	51,	52,	59,	61,	62,	64,	153	>;																											
Viz[64]=<52,60,	61,	62,	63,	65,	66,	68,	69,	113>;																										
Viz[65]=<64,66,	67,	77,	113,	118>;																													
Viz[66]=<64,65,	67,	68	>;																																
Viz[67]=<54,65,	66,	68,	69,	77,	78,	118>;																												
Viz[68]=<64,66,	67,	69	>;																																
Viz[69]=<52,53,	54,	64,	67,	68>;																														
Viz[70]=<42,53,	54,	71,	72>;																															
Viz[71]=<54,70,	72,	74,	78,	82	>;																														
Viz[72]=<42,43,	70,	71,	74>;																															
Viz[73]=<43,44,	45,	74>;																																
Viz[74]=<43,44,	45,	46,	47,	71,	72,	73,	75,	82,	83,	84,	108,	109,	112	>;																			
Viz[75]=<74,84,	108>;																																	
Viz[76]=<80,81,	86,	87,	122,123	>;																													
Viz[77]=<65,67,	78,	80,	118,121,122	>;
Viz[78]=<54,67,	71,	77,	79,	80,	81,	82>;																											
Viz[79]=<78,80,	81	>;																														
Viz[80]=<76,77,	78,	79,	81,	122>;																													
Viz[81]=<76,78,	79,	80,	82,	83,	86>;																												
Viz[82]=<71,74,	78,	81,	83,	84>;
Viz[83]=<74,81,	82,	84,	86>;																																	
Viz[84]=<74,75,	82,	83,	85,	86,	87,	108>;																														
Viz[85]=<84,87,	88,	104,	108,	110>;																																
Viz[86]=<76,81,	83,	84,	87,	123	>;																															
Viz[87]=<76,84,	85,	86,	88,	123>;																															
Viz[88]=<85,87,	104,	123,	132,	134>;																																
Viz[89]=<90,91	>;																																		
Viz[90]=<89,91,	92>;																																	
Viz[91]=<89,90,	92,	93,	94,	97,	98>;																													
Viz[92]=<90,91,	93>;																																	
Viz[93]=<91,92,	94>;																																	
Viz[94]=<91,93,	95,	98,	99>;																															
Viz[95]=<94,96,	99	>;																																	
Viz[96]=<95	>;																																				
Viz[97]=<91,98,	101,102>;																																		
Viz[98]=<91,94,	97,	99,	100,	101>;																															
Viz[99]=<94,95,	98,	100>;																																		
Viz[100]=<98,99,101>;																																		
Viz[101]=<97,98,100,102	>;																																
Viz[102]=<97,101>;																																		
Viz[103]=<3,155>;																																		
Viz[104]=<85,88,105,107,110,111,112,132	>;																							
Viz[105]=<104,106,107,111,112>;																																
Viz[106]=<105,107,133,137>;																																	
Viz[107]=<104,105,106,132,133>;																															
Viz[108]=<74,75,84,	85,	109,110,112	>;																													
Viz[109]=<74,108,112>;																																		
Viz[110]=<85,104,108,111,112>;																																
Viz[111]=<104,105,110,112>;																																		
Viz[112]=<47,74,104,105,108,109,110,111	>;																											
Viz[113]=<32,36,60,	64,	65,	114,115,116,117,118,120,125,126,127,129,137,140>;												
Viz[114]=<113,115,118>;																																		
Viz[115]=<113,114,116,118>;																																
Viz[116]=<113,115,118,119,120>;																											
Viz[117]=<113,120>;																																	
Viz[118]=<65,67,77,	113,114,115,116,119,121>;																					
Viz[119]=<116,118,120,121>;																													
Viz[120]=<113,116,117,119,121,135,137>;																							
Viz[121]=<77,118,119,120,122,123,135>;																							
Viz[122]=<76,77,80,	121,123	>;																													
Viz[123]=<76,86,87,	88,	121,122,134,135	>;																								
Viz[124]=<29,126>;																																	
Viz[125]=<29,32,113,126>;																														
Viz[126]=<29,113,124,125,127,130>;																									
Viz[127]=<113,126,128,129,130>;																											
Viz[128]=<127,129,130>;																															
Viz[129]=<113,127,128,130,140,142,149,150>;																					
Viz[130]=<126,127,128,129,131,149,150>;																								
Viz[131]=<130,150>;																																		
Viz[132]=<88,104,107,133,134>;																												
Viz[133]=<106,107,132,134,135,136,137>;																								
Viz[134]=<88,123,132,133,135>;																												
Viz[135]=<120,121,123,133,134,137>;																										
Viz[136]=<133,137>;																																		
Viz[137]=<106,113,	120,	133,	135,	136,	138,	139,	140>;																				
Viz[138]=<137,139>;																																		
Viz[139]=<137,138,140,141,142>;																												
Viz[140]=<113,129,137,139,141,142,143>;																								
Viz[141]=<139,140,142,143>;																														
Viz[142]=<129,139,140,141,143,144,145,146,149>;																				
Viz[143]=<140,141,142>;																																
Viz[144]=<142,145,149>;																																
Viz[145]=<142,144,146,147,149>;																												
Viz[146]=<142,145,147>;																																
Viz[147]=<145,146,148,149>;																														
Viz[148]=<147,149>;																																		
Viz[149]=<129,130,142,144,145,147,148,150,151>;																				
Viz[150]=<129,130,131,149>;																														
Viz[151]=<149>;																																				
Viz[152]=<27,28,29>;																																	
Viz[153]=<11,37,48,	49,	50,	63>;																														
Viz[154]=<38,39,40,	48,	52,	53>;																														
Viz[155]=<3,37,	38,	39,	40,	41,	103>;																													


// tempo computes how long the chain takes to run
decl tempo=timer();
	
decl Y = loadmat("DengueData.txt");	  // reading the data
decl T = rows(Y), n = columns(Y);
decl W = loadmat("carrega_W.txt");
decl tpop = loadmat("populacao.txt"); // reading the population size of each district
decl Ind=zeros(T,n), Ind2=zeros(T,n),aux=zeros(T,n),aux2=zeros(T,n),aux3=zeros(T,n),aux4=zeros(T,n),aux5=zeros(T,n);
decl aux6=zeros(T,n), aux7=zeros(n-1,1),miZ=zeros(T,n),VZ=zeros(T,n),Zprop=zeros(T,n),miS=zeros(T,n),VS=zeros(T,n),Sprop=zeros(T,n);
decl quad1=zeros(T,n),quad2=zeros(T,n),tauZ,tauS,aux8=zeros(T,1),aux9=zeros(T,1),aux10=zeros(T,1),aux11=zeros(T,1);
decl i,j,s,t;
decl pop = tpop';
decl theta, Indicad;
decl migama0, sigmagama0, migama1, sigmagama1, mialfa0, sigmaalfa0;

//Computing the expected values of dengue fever for each district and time

decl e=ones(T,n);

for(i=0;i<n;i++)
{
	for(t=0;t<T;t++)
	{
	e[t][i] = (sumr(Y[t][])/sumr(pop))*pop[i];
	}
}


// these are used when computing Z, S, sigmaZ e sigmaS

decl mi=zeros(T,n), u=zeros(n,1);
decl neig=zeros(n,1);

//creating an indicator matrix for the presence of dengue at time t
for (t=0;t<T;t++)
{
	for(i=0;i<n;i++)
	{
		if (Y[t][i]>0) Ind[t][i]=1; else Ind[t][i]=0;
	}
}

//creating an indicator matrix for the presence of dengue at time t-1

for (i=0;i<n;i++)	  
{
	 Ind2[0][i]=1;
}


// initial value of matrix Ind2, notice that we change this value only for the positions t and i, where Y[t-1][i]=0
for (t=1;t<T;t++)
{
	for(i=0;i<n;i++)
	{
		if (Y[t-1][i]>0) Ind2[t][i]=1; else Ind2[t][i]=0;
	}
}

//Initial values
decl G=150000,g,gama0=zeros(G,1),gama1=zeros(G,1),alfa0=zeros(G,1),sigmaS;
decl  sigmaSsample=zeros(G,1) ;
gama0[0]=2;
gama1[0]=2;
alfa0[0]=2;
sigmaS=0.01;
sigmaSsample[0]=sigmaS;


decl Z=ones(T,n);
decl thetaest=ones(T,n);
decl temp2;
decl temp1=ones(T,n);


// Starting values for Z and S


decl S = zeros(T,n);
decl Sestrela = rann(T,n);
decl Sbarra = zeros(T,1);


for (t=0;t<T;t++)
{

	Sbarra[t] = meanr(Sestrela[t][]);
	S[t][] = Sestrela[t][] - Sbarra[t];
}									 


// Computing the number of neighbors
for (i=0;i<n;i++)
{
	neig[i] = sizec(Viz[i]);
}
  	 
//burn-in and thinning
decl burnin = 50000;
decl thin = 100;
decl h=0;
decl f;

//Some starting values for the predictive


decl guarda_S = new array [((G - burnin)/thin)];

decl thetalinha = new array [((G - burnin)/thin)];
decl lambdalinha = new array [((G - burnin)/thin)];
decl Indlinha = new array [((G - burnin)/thin)];
decl Indicadoral = new array [((G - burnin)/thin)];
decl Ylinha = new array [((G - burnin)/thin)];
decl guarda_Z = new array [((G - burnin)/thin)];
decl aux14 = zeros(T,n);
decl aux15 = zeros(T,n);
decl desvio = zeros(G,1);
decl logver;

//starting the Gibbs sampler
for (g=1;g<G;g++)
{
println (g);

// computing prior means

 
for(t=0;t<T;t++)
{
	for (s=0;s<n;s++)
	{
		
		mi[t][s] = 0;
		for (i=0;i<(neig[s]);i++)
		{
		mi[t][s] = mi[t][s] + (S[t][(Viz[s][i])]);
		}
		mi[t][s] = mi[t][s]/neig[s];
	}
}
	
	/********************
		  sampling gama0
	********************/
	
	//Prior for gama0
	migama0 = 4.9;
	sigmagama0 = 0.1;
	
	decl x1, ninit = 4, dometrop=0,nsamp=1,mxsamp=zeros(1,nsamp),answercol=zeros(G,1),answer=zeros(1,1), convex=ones(1,1), npoint=50,ncent=0,qcent=zeros(1,1),xcent=zeros(1,1),neval=zeros(1,1);
	decl mxl = zeros(1,1);mxl[0][0]=-20.0;
	decl mxr = zeros(1,1);mxr[0][0]=20.0;
	decl xprev=zeros(1,1);xprev[0][0]=0.0;
	decl xinit=zeros(1,4);xinit[0][0]=0.0;xinit[0][1]=0.5;xinit[0][2]=1;xinit[0][3]=1.5;
	decl uns=ones(T,n);
	g_gama0data = new Testedata();
	g_gama0data->set_mig0(migama0);
	g_gama0data->set_sigg0(sigmagama0);
	g_gama0data->set_Z1(uns-Z);
	g_gama0data->set_T1(T);
	g_gama0data->set_n1(n);
	g_gama0data->set_I2(Ind2);
	g_gama0data->set_g1(gama1[g-1]);
	g_gama0data->set_tem(temp1);	

	// calling the ARM function
	answer = Arms(log_lik,ninit,mxl,mxr,1,xprev,mxsamp,xinit,convex,npoint,nsamp,qcent,xcent,ncent,neval);
	xprev[0][0] = mxsamp[0][0];
	gama0[g] = mxsamp[0][0];	

	
	/********************
		 sampling  gama1
	********************/
	
	//Prior for gama1
	migama1 = 1.1;
	sigmagama1 = 0.1;
	
	g_gama1data = new Testedata();
	g_gama1data->set_mig1(migama1);
	g_gama1data->set_sigg1(sigmagama1);
	g_gama1data->set_I(Ind);
	g_gama1data->set_I2(Ind2);
	g_gama1data->set_g0(gama0[g]);
	g_gama1data->set_Z1(uns-Z);
	g_gama1data->set_tem(temp1);		
	
	decl x2,answer2=zeros(1,1),xprev2=zeros(1,1),mxsamp2=zeros(1,nsamp);
	decl xinit2=zeros(1,4);xinit2[0][0]=0.0;xinit2[0][1]=2.0;xinit2[0][2]=4.0;xinit2[0][3]=7.0;
	xprev2[0][0]=0.0;
	//calling the ARM function
	answer2 = Arms(log_lik2,ninit,mxl,mxr,1,xprev2,mxsamp2,xinit2,convex,npoint,nsamp,qcent,xcent,ncent,neval);
	xprev2[0][0]= mxsamp2[0][0];
	gama1[g]= mxsamp2[0][0];
	


	/********************
		 sampling  alfa0
	********************/	
	
	//Prior for alfa0
	mialfa0 = 0;
	sigmaalfa0 = 100;
	
	decl x3, mxsamp3=zeros(1,nsamp),answer3=zeros(1,1);
	decl xprev3=zeros(1,1);xprev3[0][0]=0.0;
	decl xinit3=zeros(1,4);xinit3[0][0]=0.0;xinit3[0][1]=2.0;xinit3[0][2]=4.0;xinit3[0][3]=7.0;
	g_alfa0data = new Testedata();
	g_alfa0data->set_mia0(mialfa0);
	g_alfa0data->set_siga0(sigmaalfa0);
	g_alfa0data->set_I(Ind);
	g_alfa0data->set_obs(Y);
	g_alfa0data->set_S1(S);
	g_alfa0data->set_tem(temp1);
	g_alfa0data->set_ee(e);
	g_alfa0data->set_Z1(uns-Z);
	//calling the ARM function
	answer3 = Arms(log_lik3,ninit,mxl,mxr,1,xprev3,mxsamp3,xinit3,convex,npoint,nsamp,qcent,xcent,ncent,neval);
	xprev3[0][0] = mxsamp3[0][0];
	alfa0[g] = mxsamp3[0][0]; 

 
 
/********************
	  sampling sigmaS
********************/
decl a2,b2,alinha2,blinha2;

for (t=0;t<T;t++)
{
	for (i=0;i<n;i++)
	{
	quad2[t][i] = neig[i]*(S[t][i] - mi[t][i])^2;
	}
}

//prior for sigmaS:
a2 = 2;
b2 = 2;

//posterior full conditional  for sigmaS:
alinha2 = (n*T/2) + a2;
blinha2 = b2 + sumr(sumc(quad2/2));

tauS = rangamma(1,1,alinha2,blinha2);
sigmaS = 1/tauS;

sigmaSsample[g] = sigmaS;


// Updating the values of  sigmaZ and sigmaS
												
for (s=0;s<n;s++)
{
	u[s] = sigmaS/(neig[s]);
}


/********************
	   sampling S[t][s]
********************/


decl logpsprop,logps,logg;

for(t=0;t<T;t++)
{
	for (s=0;s<n;s++)
	{
		mi[t][s] = 0;
		for (i=0;i<(neig[s]);i++)
		{
		 	mi[t][s] = mi[t][s] + (S[t][(Viz[s][i])]);
		}

		
	mi[t][s] = mi[t][s]/neig[s];
	Sprop[t][s] = mi[t][s]+ rann(1,1)*sqrt(u[s]);

	logpsprop =  (Z[t][s])*(-exp(alfa0[g]+Sprop[t][s])*e[t][s] + Y[t][s]*Sprop[t][s]) ;
	logps =  (Z[t][s])*(-exp(alfa0[g]+S[t][s])*e[t][s] + Y[t][s]*S[t][s]) ;

 	//Metropolis ratio
	logg = (logpsprop  - logps);
	if((log(ranu(1,1)))<logg)
		S[t][s] = Sprop[t][s];
	else
		S[t][s] = S[t][s];

	}
}

for(t=0;t<T;t++)
{
	Sbarra[t] = meanr(S[t][]);
	S[t][] = S[t][] - Sbarra[t];
}

	//computing theta[t][s] to sample the indicator latent variable
	for(t=0;t<T;t++)
	{
		for (i=0;i<n;i++)
		{
			if (t>0){
				if (Y[t-1][i]>0) Ind2[t][i]=1;
			  		else Ind2[t][i]=Z[t-1][i];
			}
			if (Y[t][i]==0){
				thetaest[t][i] = (exp(gama0[g] + gama1[g]*Ind2[t][i]))/(1 + exp(gama0[g] + gama1[g]*Ind2[t][i]));
				thetaest[t][i]=thetaest[t][i]*(exp(-exp(alfa0[g]+Sprop[t][i]+log(e[t][i]))))/((thetaest[t][i]*(exp(-exp(alfa0[g]+Sprop[t][i]+log(e[t][i]))))+(1-thetaest[t][i])));
				Z[t][i] = ranbinomial(1,1,1,thetaest[t][i]);
				temp1[t][i] = Z[t][i];
				Ind[t][i]=Z[t][i];
			}	
			else
				Z[t][i]=1;			
		}
	}

	//storing sampled values after burnin and thinning
	f =  imod ((g+1),thin);
	if ((f<=0 && (g > burnin)))
	{
	
	thetalinha[h] = zeros(T,n);
	lambdalinha[h] = zeros(T,n);
	Indlinha[h] = zeros(T,n);
	Indicadoral[h] = zeros(T,n);
	Ylinha[h] = zeros(T,n);

	
	//Sampling replicate value of Y[t][s]

	lambdalinha[h] = exp(alfa0[g] + S + log(e));
	
	for (s=0;s<n;s++)
	{
	thetalinha[h][0][s] = (exp(gama0[g] + gama1[g]))/(1 + exp(gama0[g] + gama1[g]));
	Indicadoral[h][0][s] = ranbinomial(1,1,1,thetalinha[h][0][s]);
	Ylinha[h][0][s] = Indicadoral[h][0][s]*ranpoisson(1,1,lambdalinha[h][0][s]);
	}

	for (s=0;s<n;s++)
 	{
  		for (t=1;t<T;t++)
  		{
   			if (Ylinha[h][t-1][s]>0) Indlinha[h][t][s]=1; else Indlinha[h][t][s]=Z[t][s];
			thetalinha[h][t][s] = (exp(gama0[g] + Indlinha[h][t][s]*gama1[g]))/(1 + exp(gama0[g] + Indlinha[h][t][s]*gama1[g]));
			Indicadoral[h][t][s] = ranbinomial(1,1,1,thetalinha[h][t][s]);
			Ylinha[h][t][s] = Indicadoral[h][t][s]*ranpoisson(1,1,lambdalinha[h][t][s]);
  		}
 	} 
	
	guarda_S[h] = S;
	guarda_Z[h] = thetaest;

	
	h = h + 1;

	}

	// Value Imputation for districts Joa(124) and Grumari(131)
	
	for (t=0;t<52;t++)
	{
		theta = exp(gama0[g] + gama1[g]*Ind2[t][124])/(1 + exp(gama0[g] + gama1[g]*Ind2[t][124]));
		Indicad = ranbinomial(1,1,1,theta);
		Ind[t][124] = Indicad;
		Y[t][124] =  Indicad*ranpoisson(1,1,exp(alfa0[g] + S[t][124])*e[t][124]);
	}
	
	for (t=53;t<T;t++)
	{
		theta = exp(gama0[g] + gama1[g]*Ind2[t][131])/(1 + exp(gama0[g] + gama1[g]*Ind2[t][131]));
		Indicad = ranbinomial(1,1,1,theta);
		Ind[t][131] = Indicad;
		Y[t][131] =  Indicad*ranpoisson(1,1,exp(alfa0[g] + S[t][131])*e[t][131]);
	}

	for (t=1;t<52;t++)
	{
 		if (Ind[t-1][124] == 1) Ind2[t][124]= 1; else  Ind2[t][124]= 0;
		e[t][124] = (sumr(Y[t][])/sumr(pop))*pop[124];
	}

	for (t=54;t<T;t++)
	{
 		if (Ind[t-1][131] == 1) Ind2[t][131]= 1; else  Ind2[t][131]= 0;
		e[t][131] = (sumr(Y[t][])/sumr(pop))*pop[131];
	}	 

		 
	delete g_gama0data;
	delete g_gama1data;
	delete g_alfa0data;


//End of Gibbs sampler
}

println("");
// Printing results on screen

println("### gama0 ###");
print("media : ", meanc(gama0[int(G/2):]));
print("variancia : ",varc(gama0[int(G/2):]));
print("mediana : ",quantilec(gama0[int(G/2):], <1/2>));
println("Intervalo de 95% de credibilidade");
print("2,5%",quantilec(gama0[int(G/2):], <1/400>));
print("97,5%",quantilec(gama0[int(G/2):], <975/1000>));
println("");


println("### gama1 ###");
print("media : ", meanc(gama1[int(G/2):]));
print("variancia : ",varc(gama1[int(G/2):]));
print("mediana : ",quantilec(gama1[int(G/2):], <1/2>));
println("Intervalo de 95% de credibilidade");
print("2,5%",quantilec(gama1[int(G/2):], <1/400>));
print("97,5%",quantilec(gama1[int(G/2):], <975/1000>));
println("");


println("### alfa0 ###");
print("media : ", meanc(alfa0[int(G/2):]));
print("variancia : ",varc(alfa0[int(G/2):]));
print("mediana : ",quantilec(alfa0[int(G/2):], <1/2>));
println("Intervalo de 95% de credibilidade");
print("2,5%",quantilec(alfa0[int(G/2):], <1/400>));
print("97,5%",quantilec(alfa0[int(G/2):], <975/1000>));
println("");


  
println("### sigmaS ###");
print("media : ", meanc(sigmaSsample[int(G/2):]));
print("variancia : ",varc(sigmaSsample[int(G/2):]));
print("mediana : ",quantilec(sigmaSsample[int(G/2):], <1/2>));
println("Intervalo de 95% de credibilidade");
print("2,5%",quantilec(sigmaSsample[int(G/2):], <1/400>));
print("97,5%",quantilec(sigmaSsample[int(G/2):], <975/1000>));
println("");


// Tempo duracao:
println("\n Tempo computacional: ", timespan(tempo));

// saving sampled values
decl e_file = fopen("esp.txt","w");
fprintln(e_file,e);
fclose(e_file);

decl gama0_file = fopen("gama0.txt" ,"w");
fprintln(gama0_file, gama0);
fclose(gama0_file);

decl gama1_file = fopen("gama1.txt" ,"w");
fprintln(gama1_file, gama1);
fclose(gama1_file);

decl alfa0_file = fopen("alfa0.txt" ,"w");
fprintln(alfa0_file, alfa0);
fclose(alfa0_file);

decl sigmaS_file = fopen("sigmaS.txt" ,"w");
fprintln(sigmaS_file, sigmaSsample);
fclose(sigmaS_file);


// Saving replicate values after burnin and thinning
decl Temp5 = zeros(T*n,((G - burnin)/thin)), Temp6, Temp7 = zeros(T*n,((G - burnin)/thin)), Temp8;
decl Temp3 = zeros(T*n,((G- burnin)/thin)),  Temp4;

h=0;


for(g=0;g<G;g++)
{
	f =  imod ((g+1),thin);
	if ((f<=0 && (g > burnin)))
	{
	Temp5[][h]=vec(Ylinha[h]);
	Temp7[][h]=vec(guarda_Z[h]);
	Temp3[][h]=vec(guarda_S[h]);

	h = h + 1;
	}	
}

Temp6 = vec(Temp5);
decl Ylinha_file = fopen("Ylinha.txt","w");
fprintln(Ylinha_file, Temp6);
fclose(Ylinha_file);

Temp8 = vec(Temp7);
decl Z_file = fopen("latente.txt","w");
fprintln(Z_file, Temp8);
fclose(Z_file);

Temp4 = vec(Temp3);
decl S_file = fopen("efeitosS.txt","w");
fprintln(S_file, Temp4);
fclose(S_file);

//End
}
