/*
This is the Gibbs sampler for the rainfall dataset analyzed in Fernandes, Schmidt and Migon "Modelling zero-inflated spatio-temporal processes".
Here we assume that rainfall follows a Gamma distribution (see paper for details)
Notice that the fixed parameters are sampled via a ARM step. Therefore, one needs the library arms.h to run this code.
*/

#include <oxstd.h>   
#include <oxprob.h>	 
#include <oxfloat.h> 
#include <oxdraw.h>
#include "arms.h"

// Funo para usar no aquecimento da varincia
resto (const x, const y)
{
decl r;
r = (x - (floor(x/y) * y));
return r;
}

// Funo que gera uma observao da Normal N-variada
rannm(media, var)
 {
  decl N=rows(var), p, ran, nm;
  p   = choleski(var);
  ran = rann(N,1);
  nm  = p*ran + media;
  return(nm);			 //retorna uma matriz Nx1 
 }


// Estou fazendo gama0, gama1, alfa0 com ARMS e o resto com metropolis

// Chamando funao em C para gama0
decl g_gama0data;
decl g_gama1data;
decl g_alfa0data;

class Testedata
{
decl mig0,sigg0,I, I2,a0, g0, g1, Z1, S1, obs, mig1, sigg1, mia0, siga0, delt;
log_lik(const x1);
log_lik2(const x2);
log_lik3(const x3);
set_mig0(migama0);
set_sigg0(sigmagama0);
set_mig1(migama1);
set_sigg1(sigmagama1);
set_mia0(mialfa0);
set_siga0(sigmaalfa0);
set_I(Ind);
set_I2(Ind2);
set_a0(alfa0);
set_g0(gama0);
set_g1(gama1);
set_delt(delta);
set_Z1(Z);
set_S1(S);
set_obs(Y);
Estimate(const ninit, const mxl, const mxr, const domedrop, const xprev, const mxsamp);
}

Testedata::log_lik(const x1)
{
return (-(1/(2*sigg0))*(x1-mig0)^2) + (x1*sumr(sumc(I))) - sumr(sumc(log(1 + exp(x1 + g1*I2 + Z1))));
}

Testedata::log_lik2(const x2)
{
return (-(1/(2*sigg1))*(x2-mig1)^2) + (x2*sumr(sumc(I.*I2))) - sumr(sumc(log(1 + exp(g0 + x2*I2 + Z1))));
}

Testedata::log_lik3(const x3)
{
return (-(1/(2*siga0))*(x3-mia0)^2) - (x3*delt*sumr(sumc(I))) - delt*sumr(sumc(obs.*exp(-(x3+S1))));
}

Testedata::set_mig0(migama0)
{mig0 = migama0;}
Testedata::set_sigg0(sigmagama0)
{sigg0 = sigmagama0;}
Testedata::set_mig1(migama1)
{mig1 = migama1;}
Testedata::set_sigg1(sigmagama1)
{sigg1 = sigmagama1;}
Testedata::set_mia0(mialfa0)
{mia0 = mialfa0;}
Testedata::set_siga0(sigmaalfa0)
{siga0 = sigmaalfa0;}
Testedata::set_I(Ind)
{I = Ind;}
Testedata::set_I2(Ind2)
{I2 = Ind2;}
Testedata::set_g0(gama0)
{g0 = gama0;}
Testedata::set_a0(alfa0)
{a0 = alfa0;}
Testedata::set_g1(gama1)
{g1 = gama1;}
Testedata::set_delt(delta)
{delt = delta;}
Testedata::set_Z1(Z)
{Z1 = Z;}
Testedata::set_S1(S)
{S1 = S;}
Testedata::set_obs(Y)
{obs = Y;}

log_lik(const x1)
{
return g_gama0data -> log_lik(x1);
}

log_lik2(const x2)
{
return g_gama1data -> log_lik2(x2);
}

log_lik3(const x3)
{
return g_alfa0data -> log_lik3(x3);
}


//incio do programa
main ()
{
ranseed(98639963246);


// tempo computes the duration of the program
decl tempo=timer();


decl Ytot = loadmat("RainfallData.txt");
decl Y = Ytot[0:70][]; // leaving the last 3 observations out of the sample to do temporal prediction
decl T = rows(Y), n = columns(Y);
decl Ind=zeros(T,n), Ind2=zeros(T,n),aux=zeros(T,n),aux2=zeros(T,n),aux3=zeros(T,n),aux4=zeros(n-1,1),aux5=zeros(n-1,1);
decl aux6=zeros(n-1,1), aux7=zeros(n-1,1),miZ=zeros(T,n),VZ=zeros(T,n),Zprop=zeros(T,n),miS=zeros(T,n),VS=zeros(T,n),Sprop=zeros(T,n);
decl quad1=zeros(T,1),quad2=zeros(T,1),tau1,tau2,aux8=zeros(T,1),aux9=zeros(T,1),aux10=zeros(T,1),aux11=zeros(T,1),aux14=zeros(T,n);
decl leste,norte,leste2,norte2,i,j,s,t,D=zeros(n,n);


decl T1=zeros(n-1,n-1),logpZprop,logpZ,logf, T2=zeros(n-1,n-1), T3=zeros(n-1,n-1);
decl TEMP = zeros(n-1,n-1);
decl T4=zeros(n-1,n-1),logpsprop,logps,logg, T5=zeros(n-1,n-1), T6=zeros(n-1,n-1);
decl TEMP2 = zeros(n-1,n-1);

//Coordinates (in utm) of the monitoring stations

leste = <681.1,687.88,679.93,682.33,684.94,685.65,677.62,683.87,677.03,670.38,670.74,657.4,673.49,667.53,
685.21,682.09,674.2,667.91,662.79,645.7,645.31,634.8,676.64,663.89,651.76,647.49,632.6,681.42,651.58,661.43,685.92,682.39>;

norte = < 7456.22,7460.3,7456.95,7462.9,7462.91,7456.86,7463.77,7475.52,7472.69,7469.6,7474.68,7468.22,7467.47,
7465.45,7466.69,7458.38,7455.42,7461.59,7458.31,7449.95,7470.54,7466.08,7467.59,7474.74,7454.06,7466.43,
7459.35,7460.87,7475.21,7454.57,7462.18,7466.81>;


//k is the number of periods to forecast in time
decl k = 3;

//Matrix of the distances among locations

for (i=0;i<n;i++)
{
	for(j=0;j<n;j++)
	{
		D[i][j] = sqrt((leste[i] - leste[j])^2 + (norte[i] - norte[j])^2);
	}
}	



//Indicating  matrix of presence of rainfall at time t
for (t=0;t<T;t++)
{
	for(s=0;s<n;s++)
	{
		if (Y[t][s]>0) Ind[t][s]=1; else Ind[t][s]=0;
	}
}

//Indicating  matrix of presence of rainfall at time t-1

for (s=0;s<n;s++)	   //Estou forando Y[0,s]>0 para todo s
{
	 Ind2[0][s]=1;
}

for (t=1;t<T;t++)
{
	for(s=0;s<n;s++)
	{
		if (Y[t-1][s]>0) Ind2[t][s]=1; else Ind2[t][s]=0;
	}
}


//Valores iniciais 
decl G=255000;	// Number of iterations
decl g;		   // Gibbs iteration
decl gama0=zeros(G,1),gama1=zeros(G,1),alfa0=zeros(G,1),S=constant(0.5,T,n),sigma1,sigma2,phi1=zeros(G,1),phi2=zeros(G,1), Z=constant(0.5,T,n);
decl sigma1sample=zeros(G,1), sigma2sample=zeros(G,1),delta=zeros(G,1);
gama0[0]=0;
gama1[0]=0;
alfa0[0]=0;
phi1[0]=0.5;
phi2[0]=0.5;
sigma1=1;
sigma2=1;
sigma1sample[0]=sigma1;
sigma2sample[0]=sigma2;
delta[0] = 8;


decl R=zeros(n,n),W=zeros(n,n),SIGMA=zeros(n,n),PSI=zeros(n,n),IR=zeros(n,n),IW=zeros(n,n);

decl migama0,sigmagama0,gama0prop,logpgama0prop,logpgama0,loga;
decl migama1,sigmagama1,gama1prop,logpgama1prop,logpgama1,logb;
decl mialfa0,sigmaalfa0,alfa0prop,logpalfa0prop,logpalfa0,logc;

//burn-in and thinning
decl burnin = 55000;
decl thin = 200;
decl h=0;
decl f;

//variables for the tunning of the variance of the proposal
decl contador = 0;
decl V=zeros(101,1),indicad=zeros(100,1);
V[0] = 0.1;

//some starting values for the predictive distribution
decl Zlinha = new array [((G - burnin)/thin)];
decl Slinha = new array [((G - burnin)/thin)];
decl thetalinha = new array [((G - burnin)/thin)];
decl lambdalinha = new array [((G - burnin)/thin)];
decl Indlinha = new array [((G - burnin)/thin)];
decl Indicadoral = new array [((G - burnin)/thin)];
decl Ylinha = new array [((G - burnin)/thin)];
decl guarda_S = new array [((G - burnin)/thin)];
decl guarda_Z = new array [((G - burnin)/thin)];


decl Indtemp = zeros(n,1);
decl Indtemp2 = zeros(k,n);
decl media = zeros(1,32);

for(s=0;s<n;s++)
	{
		if (Y[T-1][s]>0) Indtemp[s]=1; else Indtemp[s]=0;
	}



// starting Gibbs sampler
for (g=1;g<G;g++)
{

	println (g);

	//computing Matrix SIGMA
	R = exp(-phi1[g-1]*D);
	SIGMA = sigma1*R;
	
	//computing Matrix PSI
	W = exp(-phi2[g-1]*D);
	PSI = sigma2*W;

	
	//Computing matrices inverses
	IR = invertsym(R);
	IW = invertsym(W);
	
	/********************
		  Sampling gama0
	********************/
	
	//Prior for gama0
	migama0 = 0;
	sigmagama0 = 100;
	
	decl x1, ninit = 4, dometrop=0,nsamp=1,mxsamp=zeros(1,nsamp),answercol=zeros(G,1),answer=zeros(1,1), convex=ones(1,1), npoint=50,ncent=0,qcent=zeros(1,1),xcent=zeros(1,1),neval=zeros(1,1);
	decl mxl = zeros(1,1);mxl[0][0]=-20.0;
	decl mxr = zeros(1,1);mxr[0][0]=20.0;
	decl xprev=zeros(1,1);xprev[0][0]=0.0;
	decl xinit=zeros(1,4);xinit[0][0]=0.0;xinit[0][1]=0.5;xinit[0][2]=1;xinit[0][3]=1.5;
	g_gama0data = new Testedata();
	g_gama0data->set_mig0(migama0);
	g_gama0data->set_sigg0(sigmagama0);
	g_gama0data->set_I(Ind);
	g_gama0data->set_I2(Ind2);
	g_gama0data->set_g1(gama1[g-1]);
	g_gama0data->set_Z1(Z);
	// calling ARMs function
	answer = Arms(log_lik,ninit,mxl,mxr,1,xprev,mxsamp,xinit,convex,npoint,nsamp,qcent,xcent,ncent,neval);
	xprev[0][0] = mxsamp[0][0];
	gama0[g] = mxsamp[0][0];	
	
	
	/********************
		  Sampling gama1
	********************/
	
	//Prior for gama1
	migama1 = 0;
	sigmagama1 = 100;
	
	g_gama1data = new Testedata();
	g_gama1data->set_mig1(migama1);
	g_gama1data->set_sigg1(sigmagama1);
	g_gama1data->set_I(Ind);
	g_gama1data->set_I2(Ind2);
	g_gama1data->set_g0(gama0[g]);
	g_gama1data->set_Z1(Z);
	
	
	decl x2,answer2=zeros(1,1),xprev2=zeros(1,1),mxsamp2=zeros(1,nsamp);
	decl xinit2=zeros(1,4);xinit2[0][0]=0.0;xinit2[0][1]=2.0;xinit2[0][2]=4.0;xinit2[0][3]=7.0;
	xprev2[0][0]=0.0;
	// calling ARMs function
	answer2 = Arms(log_lik2,ninit,mxl,mxr,1,xprev2,mxsamp2,xinit2,convex,npoint,nsamp,qcent,xcent,ncent,neval);
	xprev2[0][0]= mxsamp2[0][0];
	gama1[g]= mxsamp2[0][0];
	  
	
	/********************
		   Sampling alpha0
	********************/	
	
	//Prior for alpha0
	mialfa0 = 0;
	sigmaalfa0 = 100;
	
	decl x3, mxsamp3=zeros(1,nsamp),answer3=zeros(1,1);
	decl xprev3=zeros(1,1);xprev3[0][0]=0.0;
	decl xinit3=zeros(1,4);xinit3[0][0]=0.0;xinit3[0][1]=2.0;xinit3[0][2]=4.0;xinit3[0][3]=7.0;
	g_alfa0data = new Testedata();
	g_alfa0data->set_mia0(mialfa0);
	g_alfa0data->set_siga0(sigmaalfa0);
	g_alfa0data->set_I(Ind);
	g_alfa0data->set_obs(Y);
	g_alfa0data->set_S1(S);
	g_alfa0data->set_delt(delta[g-1]);
	// calling ARMs function	
	answer3 = Arms(log_lik3,ninit,mxl,mxr,1,xprev3,mxsamp3,xinit3,convex,npoint,nsamp,qcent,xcent,ncent,neval);
	xprev3[0][0] = mxsamp3[0][0];
	alfa0[g] = mxsamp3[0][0];


	/********************
		  sampling sigma1
	********************/
	decl a1,b1,alinha1,blinha1;
	
	for (t=0;t<T;t++)
	{
		quad1[t] = ((Z[t][]))*IR*(Z[t][]');
	}
	
	//prior for sigma1:
	a1 = 2;
	b1 = 14;
	
	//posterior full conditional for sigma1:
	alinha1 = (n*T/2) + a1;
	blinha1 = b1 + sumc(quad1/2);
	
	tau1 = rangamma(1,1,alinha1,blinha1);
	sigma1 = 1/tau1;

	SIGMA = sigma1*exp(-phi1[g-1]*D); //Atualizando a matriz SIGMA	
	sigma1sample[g] = sigma1;
	
	/********************
		  sampling sigma2
	********************/
	decl a2,b2,alinha2,blinha2;
	
	for (t=0;t<T;t++)
	{
		quad2[t] = ((S[t][]))*IW*(S[t][]');
	}
	
	//prior for sigma2:
	a2 = 2;
	b2 = 1.5;
	
	//posterior full conditional for sigma1:
	alinha2 = (n*T/2) + a2;
	blinha2 = b2 + sumc(quad2/2);
	
	tau2 = rangamma(1,1,alinha2,blinha2);
	sigma2 = 1/tau2;
	
	PSI=sigma2*exp(-phi2[g-1]*D); //Updating PSI	
	sigma2sample[g] = sigma2;
	
	/********************
		 Sampling  phi1
	********************/
	
	decl alfa1,beta1,det1,logphi1,phi1prop,logpiphi1prop,logpiphi1,logd,signdet={" "};

	//prior for phi1
	alfa1 = 1.178;
	beta1 = 1.085384;
	
	det1 = logdet(SIGMA,signdet);

	
	//Proposal for phi1(lognormal)
	
	logphi1 = log(phi1[g-1]) + 0.1*rann(1,1);
	phi1prop = exp(logphi1);
	
	
	decl newSIGMA = sigma1*exp(-phi1prop*D);
	decl detprop = logdet(newSIGMA,signdet);
	decl InewSIGMA = invertsym(newSIGMA);
	
	
	//computing the posterior full conditional of phi1 at the proposed value
	
	for (t=0;t<T;t++)
	{
		aux8[t] = -(detprop/2) - ((Z[t][])*InewSIGMA*(Z[t][]')/2);  
	}
	
	logpiphi1prop = sumc(aux8) + ((alfa1 - 1)*log(phi1prop)) - (beta1*phi1prop);
	
	//computing the posterior full conditional of phi1 at the current value
	
	for (t=0;t<T;t++)
	{
		aux9[t] = -(det1/2) - (quad1[t]/(2*sigma1));  
	}
	
	logpiphi1 = sumc(aux9)+ ((alfa1 - 1)*log(phi1[g-1])) - (beta1*phi1[g-1]);
	
	//Computing the Metropolis ratio
	logd = (logpiphi1prop + log(phi1prop)) - (logpiphi1 + log(phi1[g-1]));
	if((log(ranu(1,1)))<logd)
		phi1[g]=phi1prop;
	else
		phi1[g]=phi1[g-1];

	SIGMA = sigma1*exp(-phi1[g]*D); // Atualizando a matriz SIGMA
	
	/********************
		 sampling  phi2
	********************/
	
	decl alfa2,beta2,det2,logphi2,phi2prop,logpiphi2prop,logpiphi2,loge;

	//prior for phi2
	alfa2 = 1.178;
	beta2 = 1.085384;
	
	det2 = logdet(PSI,signdet);

	
	//Proposal for phi2(lognormal)
	
	logphi2 = log(phi2[g-1]) + 0.1*rann(1,1);
	phi2prop = exp(logphi2);
	
	decl newPSI = sigma2*exp(-phi2prop*D);
	decl detprop2 = logdet(newPSI,signdet);
	decl InewPSI = invertsym(newPSI);
	
	
		//computing the posterior full conditional of phi2 at the proposed value
	
	for (t=0;t<T;t++)
	{
		aux10[t] = -(detprop2/2) - ((S[t][])*InewPSI*(S[t][]')/2);  
	}
	
	logpiphi2prop = sumc(aux10) + ((alfa2 - 1)*log(phi2prop)) - (beta2*phi2prop);
	
		//computing the posterior full conditional of phi2 at the current value
	
	for (t=0;t<T;t++)
	{
		aux11[t] = -(det2/2) - (quad2[t]/(2*sigma2));  
	}
	
	logpiphi2 = sumc(aux11)+ ((alfa2 - 1)*log(phi2[g-1])) - (beta2*phi2[g-1]);
	
	//Metropolis ratio
	loge = (logpiphi2prop + log(phi2prop)) - (logpiphi2 + log(phi2[g-1]));
	if((log(ranu(1,1)))<loge)
		phi2[g]=phi2prop;
	else
		phi2[g]=phi2[g-1];

	PSI = sigma2*exp(-phi2[g]*D); // Updating PSI


	
	/********************
		sampling   delta
	********************/
	decl alfadelta,betadelta,logdelta,deltaprop,aux12=zeros(T,n),logpdeltaprop,aux13=zeros(T,n),logpdelta,logh;
	
	//prior for delta
	alfadelta = 2;
	betadelta = 2;
	
	//Tunning the variance of the proposal during the first 5050 iterations of the algorithm
	if (resto(g,50)==0 && g<5050)
	  if
	    (columns(unique(delta[((contador*50)):(g-1)])) > 25)  V[contador+1]=1.15*V[contador], indicad[contador]=1,	contador=contador+1;
	  else
		if
			(columns(unique(delta[((contador*50)):(g-1)])) < 8) V[contador+1]=0.50*V[contador], indicad[contador]=1,	contador=contador+1;
	    else 
			V[contador+1]=V[contador], indicad[contador]=1,	contador=contador+1;
	
	
	if (g==5051) V[100]=sumc(V[0:99].*indicad)/sumc(indicad);	   
	
	//Proposal for delta(lognormal)
	
	logdelta = log(delta[g-1]) + V[contador]*rann(1,1);
	deltaprop = exp(logdelta);
	
	//computing the posterior full conditional of delta at the proposed value
	
	for (t=0;t<T;t++)
	{
		for (s=0;s<n;s++)
		{
			if (Y[t][s]>0) aux12[t][s] = deltaprop*log(deltaprop) - deltaprop*(alfa0[g]+S[t][s]) - log(gammafact(deltaprop)) + (deltaprop-1)*log(Y[t][s]) - deltaprop*Y[t][s]*exp(-(alfa0[g]+S[t][s])); else aux12[t][s] = 0;
		}
	}
	logpdeltaprop = (alfadelta-1)*deltaprop - betadelta*deltaprop + sumc(sumr(aux12));
	
	//computing the posterior full conditional of delta at the current value
	
	for (t=0;t<T;t++)
	{
		for (s=0;s<n;s++)
		{
			if (Y[t][s]>0) aux13[t][s] = delta[g-1]*log(delta[g-1]) - delta[g-1]*(alfa0[g]+S[t][s]) - log(gammafact(delta[g-1])) + (delta[g-1]-1)*log(Y[t][s]) - delta[g-1]*Y[t][s]*exp(-(alfa0[g]+S[t][s])); else aux13[t][s] = 0;
		}
	}
	logpdelta = (alfadelta-1)*delta[g-1] - betadelta*delta[g-1] + sumc(sumr(aux13));
	
	//Metropolis ratio
	logh = (logpdeltaprop + log(deltaprop)) - (logpdelta + log(delta[g-1]));
	if(log(ranu(1,1))<logh)
	  delta[g] = deltaprop;
	  else
 	 delta[g] = delta[g-1];

	
	/********************
		Sampling   Z[t,s]
	********************/

	// prior for Z[t,s]	
			
	T1 = invertsym(SIGMA[1:n-1][1:n-1]);
	
	for(t=0;t<T;t++)
	{
		miZ[t][0] = (SIGMA[0][1:n-1])*T1*(Z[t][1:n-1])';
		VZ[t][0] = SIGMA[0][0] - (SIGMA[0][1:n-1])*T1*(SIGMA[1:n-1][0]);

		Zprop[t][0] = miZ[t][0] + rann(1,1)*sqrt(VZ[t][0]);
	
		logpZprop = - log(1 + exp(gama0[g] + gama1[g]*Ind2[t][0] + Zprop[t][0])) + Ind[t][0]*Zprop[t][0];	
		logpZ = - log(1 + exp(gama0[g] + gama1[g]*Ind2[t][0] + Z[t][0])) + Ind[t][0]*Z[t][0];
	
	 	//Metropolis ratio
		logf = (logpZprop  - logpZ);
		if((log(ranu(1,1)))<logf)
			Z[t][0] = Zprop[t][0];
		else
			Z[t][0] = Z[t][0];
	}
  	     	
	 for (i=1;i<(n-1);i++)
	 {
	 aux4 = (SIGMA[i][0:(i-1)])~(SIGMA[i][(i+1):(n-1)]);
	 aux5 = (SIGMA[0:(i-1)][i])|(SIGMA[(i+1):(n-1)][i]);
	
	 TEMP = ((SIGMA[0:(i-1)][0:(i-1)])~(SIGMA[0:(i-1)][(i+1):(n-1)]))|((SIGMA[(i+1):(n-1)][0:(i-1)])~(SIGMA[(i+1):(n-1)][(i+1):(n-1)]));
	 T2 = invertsym(TEMP);

	 
	 	for (t=0;t<T;t++)
		{
		miZ[t][i] = (aux4)*T2*((Z[t][0:(i-1)])~(Z[t][(i+1):(n-1)]))';
		VZ[t][i] = SIGMA[i][i] - (aux4)*T2*(aux5);

		Zprop[t][i] = miZ[t][i] + rann(1,1)*sqrt(VZ[t][i]);
	
		logpZprop =  - log(1 + exp(gama0[g] + gama1[g]*Ind2[t][i] + Zprop[t][i])) + Ind[t][i]*Zprop[t][i];
		logpZ =  - log(1 + exp(gama0[g] + gama1[g]*Ind2[t][i] + Z[t][i])) + Ind[t][i]*Z[t][i];
		
	 	//Metropolis ratio
		logf = (logpZprop  - logpZ);
		if((log(ranu(1,1)))<logf)
			Z[t][i] = Zprop[t][i];
		else
			Z[t][i] = Z[t][i];
		}
	}
	
	T3 = invertsym(SIGMA[:n-2][:n-2]);
	
	for(t=0;t<T;t++)
	{
		miZ[t][n-1] = (SIGMA[n-1][:n-2])*T3*(Z[t][:n-2])';
		VZ[t][n-1] = SIGMA[n-1][n-1] - (SIGMA[n-1][:n-2])*T3*(SIGMA[:n-2][n-1]);
	
		Zprop[t][n-1] = miZ[t][n-1] + rann(1,1)*sqrt(VZ[t][n-1]);
	
		logpZprop =  - log(1 + exp(gama0[g] + gama1[g]*Ind2[t][n-1] + Zprop[t][n-1])) + Ind[t][n-1]*Zprop[t][n-1];
		logpZ =  - log(1 + exp(gama0[g] + gama1[g]*Ind2[t][n-1] + Z[t][n-1])) + Ind[t][n-1]*Z[t][n-1];
		
	 	//Utilizando Metrpolis
		logf = (logpZprop  - logpZ);
		if((log(ranu(1,1)))<logf)
			Z[t][n-1] = Zprop[t][n-1];
		else
			Z[t][n-1] = Z[t][n-1];
	}
	
	/********************
		sampling   S[t,s]
	********************/
	
	// prior for S[t,s]	
	
	T4 = invertsym(PSI[1:n-1][1:n-1]);
	
	for(t=0;t<T;t++)
	{
		miS[t][0] = (PSI[0][1:n-1])*T4*(S[t][1:n-1])';
		VS[t][0] = PSI[0][0] - (PSI[0][1:n-1])*T4*(PSI[1:n-1][0]);
		
		Sprop[t][0] = miS[t][0] + rann(1,1)*sqrt(VS[t][0]);
	
		logpsprop = - (delta[g]*Y[t][0]*exp(-(alfa0[g]+Sprop[t][0]))) - (Ind[t][0]*delta[g]*Sprop[t][0]);
		logps = - (delta[g]*Y[t][0]*exp(-(alfa0[g]+S[t][0]))) - (Ind[t][0]*delta[g]*S[t][0]);
	
	 	//Metropolis ratio
		logg = (logpsprop  - logps);
		if((log(ranu(1,1)))<logg)
			S[t][0] = Sprop[t][0];
		else
			S[t][0] = S[t][0];
	}

	
	 for (i=1;i<n-1;i++)
	 {
	 aux6 = (PSI[i][0:i-1])~(PSI[i][i+1:n-1]);
	 aux7 = (PSI[0:i-1][i])|(PSI[i+1:n-1][i]);

	 TEMP2 = ((PSI[0:i-1][0:i-1])~(PSI[0:i-1][i+1:]))|((PSI[i+1:][0:i-1])~(PSI[i+1:][i+1:]));
	 T5 = invertsym(TEMP2);
	
	 
	 	for (t=0;t<T;t++)
		{
		miS[t][i] = (aux6)*T5*((S[t][0:i-1])~(S[t][i+1:]))';
		VS[t][i] = PSI[i][i] - (aux6)*T5*(aux7);
		
		Sprop[t][i] = miS[t][i] + rann(1,1)*sqrt(VS[t][i]);
	
		logpsprop = - (delta[g]*Y[t][i]*exp(-(alfa0[g]+Sprop[t][i]))) - (Ind[t][i]*delta[g]*Sprop[t][i]);
		logps =  - (delta[g]*Y[t][i]*exp(-(alfa0[g]+S[t][i]))) - (Ind[t][i]*delta[g]*S[t][i]);
	
	 	//Utilizando Metrpolis
		logg = (logpsprop  - logps);
		if((log(ranu(1,1)))<logg)
			S[t][i] = Sprop[t][i];
		else
			S[t][i] = S[t][i];
		 }
	}
	
	
	T6 = invertsym(PSI[:n-2][:n-2]);
	
	for(t=0;t<T;t++)
	{
		miS[t][n-1] = (PSI[n-1][:n-2])*T6*(S[t][:n-2])';
		VS[t][n-1] = PSI[n-1][n-1] - (PSI[n-1][:n-2])*T6*(PSI[:n-2][n-1]);
	
		Sprop[t][n-1] = miS[t][n-1] + rann(1,1)*sqrt(VS[t][n-1]);
	
		logpsprop = - (delta[g]*Y[t][n-1]*exp(-(alfa0[g]+Sprop[t][n-1]))) - (delta[g]*Ind[t][n-1]*Sprop[t][n-1]);
		logps =  - (delta[g]*Y[t][n-1]*exp(-(alfa0[g]+S[t][n-1]))) - (delta[g]*Ind[t][n-1]*S[t][n-1]);
	
	 	//Metropolis ratio
		logg = (logpsprop  - logps);
		if((log(ranu(1,1)))<logg)
			S[t][n-1] = Sprop[t][n-1];
		else
			S[t][n-1] = S[t][n-1];
	}

	delete g_gama0data;
	delete g_gama1data;
	delete g_alfa0data;


	  // Sampling from the predictive distribution only after convergence
	f =  imod ((g+1),thin);
	if ((f<=0 && (g > burnin)))
	{
	
	
	Zlinha[h] = zeros(k,n);
	Slinha[h] = zeros(k,n);
	thetalinha[h] = zeros(k,n);
	lambdalinha[h] = zeros(k,n);
	Indlinha[h] = zeros(k,n);
	Indicadoral[h] = zeros(k,n);
	Ylinha[h] = zeros(k,n);
	guarda_S[h] = zeros(T,n);
	guarda_Z[h] = zeros(T,n);

	// Saving samples from Z and S
	guarda_Z[h][][] = Z[][];
	guarda_S[h][][] = S[][];
	
	// first temporal prediction
	
	Zlinha[h][0][] = rannm(media,SIGMA);
	Slinha[h][0][] = rannm(media,PSI);

 	for (s=0;s<n;s++)
 	{	
	thetalinha[h][0][s] = (exp(gama0[g] + gama1[g]*Indtemp[s] + Zlinha[h][0][s]))/(1 + exp(gama0[g] + gama1[g]*Indtemp[s] + Zlinha[h][0][s]));
	Indicadoral[h][0][s] = ranbinomial(1,1,1,thetalinha[h][0][s]);
	lambdalinha[h][0][s] = exp(-(alfa0[g] + Slinha[h][0][s]));
	Ylinha[h][0][s] = Indicadoral[h][0][s]*rangamma(1,1,delta[g],(delta[g]*lambdalinha[h][0][s]));
	}

	// next k-1 predictions
	for (t=1;t<k;t++)
	{
		Zlinha[h][t][] = rannm(media,SIGMA);
		Slinha[h][t][] = rannm(media,PSI);


		for (s=0;s<n;s++)
 		{
			if (Ylinha[h][t-1][s]>0) Indtemp2[t][s]=1; else Indtemp2[t][s]=0;

			thetalinha[h][t][s] = (exp(gama0[g] + gama1[g]*Indtemp2[t][s] + Zlinha[h][t][s]))/(1 + exp(gama0[g] + gama1[g]*Indtemp2[t][s] + Zlinha[h][t][s]));
			Indicadoral[h][t][s] = ranbinomial(1,1,1,thetalinha[h][t][s]);
			lambdalinha[h][t][s] = exp(-(alfa0[g] + Slinha[h][t][s]));
			Ylinha[h][t][s] = Indicadoral[h][t][s]*rangamma(1,1,delta[g],(delta[g]*lambdalinha[h][t][s]));
	   	}
	}

	
	h = h + 1;

	
	//fim do if
	} 



	
// end of gibbs sampler
}



 
println("");

// Printing results on screen

println("### gama0 ###");
print("media : ", meanc(gama0[int(G/2):]));
print("variancia : ",varc(gama0[int(G/2):]));
print("mediana : ",quantilec(gama0[int(G/2):], <1/2>));
println("Intervalo de 95% de credibilidade");
print("2,5%",quantilec(gama0[int(G/2):], <1/400>));
print("97,5%",quantilec(gama0[int(G/2):], <975/1000>));
println("");


println("### gama1 ###");
print("media : ", meanc(gama1[int(G/2):]));
print("variancia : ",varc(gama1[int(G/2):]));
print("mediana : ",quantilec(gama1[int(G/2):], <1/2>));
println("Intervalo de 95% de credibilidade");
print("2,5%",quantilec(gama1[int(G/2):], <1/400>));
print("97,5%",quantilec(gama1[int(G/2):], <975/1000>));
println("");


println("### alfa0 ###");
print("media : ", meanc(alfa0[int(G/2):]));
print("variancia : ",varc(alfa0[int(G/2):]));
print("mediana : ",quantilec(alfa0[int(G/2):], <1/2>));
println("Intervalo de 95% de credibilidade");
print("2,5%",quantilec(alfa0[int(G/2):], <1/400>));
print("97,5%",quantilec(alfa0[int(G/2):], <975/1000>));
println("");



println("### phi1 ###");
print("media : ", meanc(phi1[int(G/2):]));
print("variancia : ",varc(phi1[int(G/2):]));
print("mediana : ",quantilec(phi1[int(G/2):], <1/2>));
println("Intervalo de 95% de credibilidade");
print("2,5%",quantilec(phi1[int(G/2):], <1/400>));
print("97,5%",quantilec(phi1[int(G/2):], <975/1000>));
println("");


println("### phi2 ###");
print("media : ", meanc(phi2[int(G/2):]));
print("variancia : ",varc(phi2[int(G/2):]));
print("mediana : ",quantilec(phi2[int(G/2):], <1/2>));
println("Intervalo de 95% de credibilidade");
print("2,5%",quantilec(phi2[int(G/2):], <1/400>));
print("97,5%",quantilec(phi2[int(G/2):], <975/1000>));
println("");

  
println("### sigma1 ###");
print("media : ", meanc(sigma1sample[int(G/2):]));
print("variancia : ",varc(sigma1sample[int(G/2):]));
print("mediana : ",quantilec(sigma1sample[int(G/2):], <1/2>));
println("Intervalo de 95% de credibilidade");
print("2,5%",quantilec(sigma1sample[int(G/2):], <1/400>));
print("97,5%",quantilec(sigma1sample[int(G/2):], <975/1000>));
println("");


println("### sigma2 ###");
print("media : ", meanc(sigma2sample[int(G/2):]));
print("variancia : ",varc(sigma2sample[int(G/2):]));
print("mediana : ",quantilec(sigma2sample[int(G/2):], <1/2>));
println("Intervalo de 95% de credibilidade");
print("2,5%",quantilec(sigma2sample[int(G/2):], <1/400>));
print("97,5%",quantilec(sigma2sample[int(G/2):], <975/1000>));
println("");

println("### delta ###");
print("media : ", meanc(delta[int(G/2):]));
print("variancia : ",varc(delta[int(G/2):]));
print("mediana : ",quantilec(delta[int(G/2):], <1/2>));
println("Intervalo de 95% de credibilidade");
print("2,5%",quantilec(delta[int(G/2):], <1/400>));
print("97,5%",quantilec(delta[int(G/2):], <975/1000>));
println("");


// Time
println("\n Tempo computacional: ", timespan(tempo));


///////////////////////////////


//Exporting chains to files as cadeias
decl gama0_file = fopen("gama0.txt" ,"w");
decl gama1_file = fopen("gama1.txt" ,"w");
decl alfa0_file = fopen("alfa0.txt" ,"w");
decl phi1_file = fopen("phi1.txt" ,"w");
decl phi2_file = fopen("phi2.txt" ,"w");
decl sigma1_file = fopen("sigma1.txt" ,"w");
decl sigma2_file = fopen("sigma2.txt" ,"w");
decl delta_file = fopen("delta.txt" ,"w");


// Se quiser salvar todas as amostras dos parmetros fixos usa isso


		fprintln(gama0_file, gama0);
		fprintln(gama1_file, gama1);
		fprintln(alfa0_file, alfa0);
		fprintln(phi1_file, phi1);
		fprintln(phi2_file, phi2);
		fprintln(sigma1_file, sigma1sample);
		fprintln(sigma2_file, sigma2sample);
		fprintln(delta_file, delta);


// Caso contrrio usa isso

/*
for (g=0;g<G;g++)
{
	f = imod ((g+1),thin);
	if ((f<=0 && (g > burnin)))
	{
		fprintln(gama0_file, gama0[g]);
		fprintln(gama1_file, gama1[g]);
		fprintln(alfa0_file, alfa0[g]);
		fprintln(phi1_file, phi1[g]);
		fprintln(phi2_file, phi2[g]);
		fprintln(sigma1_file, sigma1sample[g]);
		fprintln(sigma2_file, sigma2sample[g]);
		fprintln(delta_file, delta[g]);

		
	}
}		  */

fclose(gama0_file);
fclose(gama1_file);
fclose(alfa0_file);
fclose(phi1_file);
fclose(phi2_file);
fclose(sigma1_file);
fclose(sigma2_file);
fclose(delta_file);

/////////////////////////////////////////////



// Exporting predicted values
decl Temp1 = zeros(k*T,((G - burnin)/thin)), Temp2;
decl Temp3 = zeros(T*n,((G - burnin)/thin)), Temp4, Temp5 = zeros(T*n,((G - burnin)/thin)), Temp6;
h=0;


//Saving samples from prediction, S and Z after burn-in and thinning
for(g=0;g<G;g++)
{
	f =  imod ((g+1),thin);
	if ((f<=0 && (g > burnin)))
	{
		Temp1[][h]=vec(Ylinha[h][][]);
		Temp3[][h]=vec(guarda_S[h][][]);
		Temp5[][h]=vec(guarda_Z[h][][]);
		
				
		h = h + 1;
	}	
}

Temp2 = vec(Temp1);
Temp4 = vec(Temp3);
Temp6 = vec(Temp5);


decl Ylinha_file = fopen("Ylinha.txt","w");
fprintln(Ylinha_file, Temp2);
fclose(Ylinha_file);

decl guarda_S_file = fopen("guarda_S.txt","w");
fprintln(guarda_S_file, Temp4);
fclose(guarda_S_file);

decl guarda_Z_file = fopen("guarda_Z.txt","w");
fprintln(guarda_Z_file, Temp6);
fclose(guarda_Z_file);
  
// end
}						  					